@@ -11,10 +11,11 @@
 
 namespace Cache\IntegrationTests;
 
+use PHPUnit\Framework\TestCase;
 use Psr\Cache\CacheItemInterface;
 use Psr\Cache\CacheItemPoolInterface;
 
-abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
+abstract class CachePoolTest extends TestCase
 {
     /**
      * @type array with functionName => reason.
@@ -24,19 +25,25 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     /**
      * @type CacheItemPoolInterface
      */
-    private $cache;
+    protected $cache;
 
     /**
      * @return CacheItemPoolInterface that is used in the tests
      */
     abstract public function createCachePool();
 
-    protected function setUp()
+    /**
+     * @before
+     */
+    public function setupService()
     {
         $this->cache = $this->createCachePool();
     }
 
-    protected function tearDown()
+    /**
+     * @after
+     */
+    public function tearDownService()
     {
         if ($this->cache !== null) {
             $this->cache->clear();
@@ -75,8 +82,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -106,12 +111,38 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
         $this->assertFalse($this->cache->getItem('key2')->isHit());
     }
 
+    public function testBasicUsageWithLongKey()
+    {
+        if (isset($this->skippedTests[__FUNCTION__])) {
+            $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
+        }
+
+        $pool = $this->createCachePool();
+
+        $key = str_repeat('a', 300);
+
+        $item = $pool->getItem($key);
+        $this->assertFalse($item->isHit());
+        $this->assertSame($key, $item->getKey());
+
+        $item->set('value');
+        $this->assertTrue($pool->save($item));
+
+        $item = $pool->getItem($key);
+        $this->assertTrue($item->isHit());
+        $this->assertSame($key, $item->getKey());
+        $this->assertSame('value', $item->get());
+
+        $this->assertTrue($pool->deleteItem($key));
+
+        $item = $pool->getItem($key);
+        $this->assertFalse($item->isHit());
+    }
+
     public function testItemModifiersReturnsStatic()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -124,8 +155,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -147,8 +176,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $keys  = ['foo', 'bar', 'baz'];
@@ -193,8 +220,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $items = $this->cache->getItems([]);
@@ -215,8 +240,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -234,8 +257,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -253,8 +274,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -271,8 +290,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -290,8 +307,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $items = $this->cache->getItems(['foo', 'bar', 'baz']);
@@ -321,8 +336,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -337,8 +350,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -355,8 +366,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('test_ttl_null');
@@ -375,8 +384,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -404,8 +411,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -423,8 +428,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -445,8 +448,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $this->prepareDeferredSaveWithoutCommit();
@@ -470,8 +471,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -486,12 +485,13 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($return, 'commit() should return true even if no items were deferred. ');
     }
 
+    /**
+     * @medium
+     */
     public function testExpiration()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -509,8 +509,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -526,8 +524,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -543,8 +539,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -560,8 +554,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $key  = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_.';
@@ -573,77 +565,67 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testGetItemInvalidKeys($key)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $this->cache->getItem($key);
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testGetItemsInvalidKeys($key)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $this->cache->getItems(['key1', $key, 'key2']);
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testHasItemInvalidKeys($key)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $this->cache->hasItem($key);
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testDeleteItemInvalidKeys($key)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $this->cache->deleteItem($key);
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testDeleteItemsInvalidKeys($key)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $this->cache->deleteItems(['key1', $key, 'key2']);
     }
 
@@ -651,8 +633,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -668,8 +648,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -685,8 +663,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -704,8 +680,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $float = 1.23456789;
@@ -723,11 +697,9 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
-        $item  = $this->cache->getItem('key');
+        $item = $this->cache->getItem('key');
         $item->set(true);
         $this->cache->save($item);
 
@@ -741,8 +713,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $array = ['a' => 'foo', 2 => 'bar'];
@@ -760,8 +730,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $object    = new \stdClass();
@@ -776,12 +744,29 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($item->isHit(), 'isHit() should return true when object are stored. ');
     }
 
+    public function testBinaryData()
+    {
+        if (isset($this->skippedTests[__FUNCTION__])) {
+            $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
+        }
+
+        $data = '';
+        for ($i = 0; $i < 256; $i++) {
+            $data .= chr($i);
+        }
+
+        $item = $this->cache->getItem('key');
+        $item->set($data);
+        $this->cache->save($item);
+
+        $item = $this->cache->getItem('key');
+        $this->assertTrue($data === $item->get(), 'Binary data must survive a round trip.');
+    }
+
     public function testIsHit()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -796,8 +781,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -817,8 +800,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -840,8 +821,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -864,8 +843,6 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
@@ -877,12 +854,13 @@ abstract class CachePoolTest extends \PHPUnit_Framework_TestCase
         $this->assertInstanceOf('DateTime', $value, 'You must be able to store objects in cache.');
     }
 
+    /**
+     * @medium
+     */
     public function testHasItemReturnsFalseWhenDeferredItemIsExpired()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key');
