@@ -11,9 +11,10 @@
 
 namespace Cache\IntegrationTests;
 
+use PHPUnit\Framework\TestCase;
 use Psr\SimpleCache\CacheInterface;
 
-abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
+abstract class SimpleCacheTest extends TestCase
 {
     /**
      * @type array with functionName => reason.
@@ -23,19 +24,40 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
     /**
      * @type CacheInterface
      */
-    private $cache;
+    protected $cache;
 
     /**
      * @return CacheInterface that is used in the tests
      */
     abstract public function createSimpleCache();
 
-    protected function setUp()
+    /**
+     * Advance time perceived by the cache for the purposes of testing TTL.
+     *
+     * The default implementation sleeps for the specified duration,
+     * but subclasses are encouraged to override this,
+     * adjusting a mocked time possibly set up in {@link createSimpleCache()},
+     * to speed up the tests.
+     *
+     * @param int $seconds
+     */
+    public function advanceTime($seconds)
+    {
+        sleep($seconds);
+    }
+
+    /**
+     * @before
+     */
+    public function setupService()
     {
         $this->cache = $this->createSimpleCache();
     }
 
-    protected function tearDown()
+    /**
+     * @after
+     */
+    public function tearDownService()
     {
         if ($this->cache !== null) {
             $this->cache->clear();
@@ -43,18 +65,32 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
     }
 
     /**
-     * Data provider for invalid keys.
+     * Data provider for invalid cache keys.
      *
      * @return array
      */
     public static function invalidKeys()
+    {
+        return array_merge(
+            self::invalidArrayKeys(),
+            [
+                [2],
+            ]
+        );
+    }
+
+    /**
+     * Data provider for invalid array keys.
+     *
+     * @return array
+     */
+    public static function invalidArrayKeys()
     {
         return [
             [''],
             [true],
             [false],
             [null],
-            [2],
             [2.5],
             ['{str'],
             ['rand{'],
@@ -132,21 +168,25 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals('value', $this->cache->get('key'));
     }
 
+    /**
+     * @medium
+     */
     public function testSetTtl()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
-        $result = $this->cache->set('key1', 'value', 1);
+        $result = $this->cache->set('key1', 'value', 2);
         $this->assertTrue($result, 'set() must return true if success');
         $this->assertEquals('value', $this->cache->get('key1'));
-        sleep(2);
-        $this->assertNull($this->cache->get('key1'), 'Value must expire after ttl.');
 
-        $this->cache->set('key2', 'value', new \DateInterval('PT1S'));
+        $this->cache->set('key2', 'value', new \DateInterval('PT2S'));
         $this->assertEquals('value', $this->cache->get('key2'));
-        sleep(2);
+
+        $this->advanceTime(3);
+
+        $this->assertNull($this->cache->get('key1'), 'Value must expire after ttl.');
         $this->assertNull($this->cache->get('key2'), 'Value must expire after ttl.');
     }
 
@@ -213,28 +253,38 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($result, 'setMultiple() must return true if success');
         $this->assertEquals('value0', $this->cache->get('key0'));
         $this->assertEquals('value1', $this->cache->get('key1'));
+    }
+
+    public function testSetMultipleWithIntegerArrayKey()
+    {
+        if (isset($this->skippedTests[__FUNCTION__])) {
+            $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
+        }
 
         $result = $this->cache->setMultiple(['0' => 'value0']);
         $this->assertTrue($result, 'setMultiple() must return true if success');
         $this->assertEquals('value0', $this->cache->get('0'));
     }
 
+    /**
+     * @medium
+     */
     public function testSetMultipleTtl()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
-        $this->cache->setMultiple(['key2' => 'value2', 'key3' => 'value3'], 1);
+        $this->cache->setMultiple(['key2' => 'value2', 'key3' => 'value3'], 2);
         $this->assertEquals('value2', $this->cache->get('key2'));
         $this->assertEquals('value3', $this->cache->get('key3'));
-        sleep(2);
+
+        $this->cache->setMultiple(['key4' => 'value4'], new \DateInterval('PT2S'));
+        $this->assertEquals('value4', $this->cache->get('key4'));
+
+        $this->advanceTime(3);
         $this->assertNull($this->cache->get('key2'), 'Value must expire after ttl.');
         $this->assertNull($this->cache->get('key3'), 'Value must expire after ttl.');
-
-        $this->cache->setMultiple(['key4' => 'value4'], new \DateInterval('PT1S'));
-        $this->assertEquals('value4', $this->cache->get('key4'));
-        sleep(2);
         $this->assertNull($this->cache->get('key4'), 'Value must expire after ttl.');
     }
 
@@ -369,8 +419,26 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
         $this->assertTrue($this->cache->has('key0'));
     }
 
+    public function testBasicUsageWithLongKey()
+    {
+        if (isset($this->skippedTests[__FUNCTION__])) {
+            $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
+        }
+
+        $key = str_repeat('a', 300);
+
+        $this->assertFalse($this->cache->has($key));
+        $this->assertTrue($this->cache->set($key, 'value'));
+
+        $this->assertTrue($this->cache->has($key));
+        $this->assertSame('value', $this->cache->get($key));
+
+        $this->assertTrue($this->cache->delete($key));
+
+        $this->assertFalse($this->cache->has($key));
+    }
+
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testGetInvalidKeys($key)
@@ -379,11 +447,11 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->get($key);
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testGetMultipleInvalidKeys($key)
@@ -392,23 +460,21 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $result = $this->cache->getMultiple(['key1', $key, 'key2']);
     }
 
-    /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
-     */
     public function testGetMultipleNoIterable()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $result = $this->cache->getMultiple('key');
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testSetInvalidKeys($key)
@@ -417,12 +483,12 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->set($key, 'foobar');
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
-     * @dataProvider invalidKeys
+     * @dataProvider invalidArrayKeys
      */
     public function testSetMultipleInvalidKeys($key)
     {
@@ -430,32 +496,26 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
-        if (is_int($key)) {
-            $this->markTestSkipped('As keys, strings are always casted to ints so they should be accepted');
-        }
-
         $values = function () use ($key) {
             yield 'key1' => 'foo';
             yield $key => 'bar';
             yield 'key2' => 'baz';
         };
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->setMultiple($values());
     }
 
-    /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
-     */
     public function testSetMultipleNoIterable()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->setMultiple('key');
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testHasInvalidKeys($key)
@@ -464,11 +524,11 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->has($key);
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testDeleteInvalidKeys($key)
@@ -477,11 +537,11 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->delete($key);
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testDeleteMultipleInvalidKeys($key)
@@ -490,23 +550,21 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->deleteMultiple(['key1', $key, 'key2']);
     }
 
-    /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
-     */
     public function testDeleteMultipleNoIterable()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->deleteMultiple('key');
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidTtl
      */
     public function testSetInvalidTtl($ttl)
@@ -515,11 +573,11 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->set('key', 'value', $ttl);
     }
 
     /**
-     * @expectedException \Psr\SimpleCache\InvalidArgumentException
      * @dataProvider invalidTtl
      */
     public function testSetMultipleInvalidTtl($ttl)
@@ -528,6 +586,7 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
         }
 
+        $this->expectException('Psr\SimpleCache\InvalidArgumentException');
         $this->cache->setMultiple(['key' => 'value'], $ttl);
     }
 
@@ -620,6 +679,23 @@ abstract class SimpleCacheTest extends \PHPUnit_Framework_TestCase
         $this->assertEquals($object, $result);
     }
 
+    public function testBinaryData()
+    {
+        if (isset($this->skippedTests[__FUNCTION__])) {
+            $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
+        }
+
+        $data = '';
+        for ($i = 0; $i < 256; $i++) {
+            $data .= chr($i);
+        }
+
+        $array = ['a' => 'foo', 2 => 'bar'];
+        $this->cache->set('key', $data);
+        $result = $this->cache->get('key');
+        $this->assertTrue($data === $result, 'Binary data must survive a round trip.');
+    }
+
     /**
      * @dataProvider validKeys
      */
