@@ -12,11 +12,12 @@
 namespace Cache\IntegrationTests;
 
 use Cache\TagInterop\TaggableCacheItemPoolInterface;
+use PHPUnit\Framework\TestCase;
 
 /**
  * @author Tobias Nyholm <tobias.nyholm@gmail.com>
  */
-abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
+abstract class TaggableCachePoolTest extends TestCase
 {
     /**
      * @type array with functionName => reason.
@@ -26,19 +27,25 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     /**
      * @type TaggableCacheItemPoolInterface
      */
-    private $cache;
+    protected $cache;
 
     /**
      * @return TaggableCacheItemPoolInterface that is used in the tests
      */
     abstract public function createCachePool();
 
-    protected function setUp()
+    /**
+     * @before
+     */
+    public function setupService()
     {
         $this->cache = $this->createCachePool();
     }
 
-    protected function tearDown()
+    /**
+     * @after
+     */
+    public function tearDownService()
     {
         if ($this->cache !== null) {
             $this->cache->clear();
@@ -54,8 +61,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $this->cache->save($this->cache->getItem('key1')->set('value')->setTags(['tag1', 'tag2']));
@@ -80,8 +85,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -103,8 +106,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -118,45 +119,35 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
         $this->assertCount(1, $item->getPreviousTags());
     }
 
-    /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
-     */
     public function testTagAccessorWithEmptyTag()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $item->setTags(['']);
-        $this->cache->save($item);
     }
 
     /**
-     * @expectedException \Psr\Cache\InvalidArgumentException
      * @dataProvider invalidKeys
      */
     public function testTagAccessorWithInvalidTag($tag)
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
+        $this->expectException('Psr\Cache\InvalidArgumentException');
         $item->setTags([$tag]);
-        $this->cache->save($item);
     }
 
     public function testTagAccessorDuplicateTags()
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -175,8 +166,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -199,8 +188,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -222,8 +209,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -251,8 +236,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $item = $this->cache->getItem('key')->set('value');
@@ -281,8 +264,6 @@ abstract class TaggableCachePoolTest extends \PHPUnit_Framework_TestCase
     {
         if (isset($this->skippedTests[__FUNCTION__])) {
             $this->markTestSkipped($this->skippedTests[__FUNCTION__]);
-
-            return;
         }
 
         $pool = $this->cache;
